// Copyright (c) 2009-2010 Volvox Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Author: Konstantin Lepa <konstantin.lepa@gmail.com>

#include "vxlog/format-string.h"
#include "utests/googletest.h"

namespace {

TEST(FormatString, OnePlaceHolder) {
  vxlog::FormatString fs;

  fs.Init("Hello, %_!");
  fs.Replace("World");
  EXPECT_STREQ("Hello, World!", fs.GetResult());
}

TEST(FormatString, OneSmallPlaceHolder) {
  vxlog::FormatString fs;

  fs.Init("Symbol `%_'");
  fs.Replace('c');
  EXPECT_STREQ("Symbol `c'", fs.GetResult());
}

TEST(FormatString, SimpleProcent) {
  vxlog::FormatString fs;

  fs.Init("%_, 100%!");
  fs.Replace("Ok");
  fs.Replace("Unused");
  EXPECT_STREQ("Ok, 100%!", fs.GetResult());
}

TEST(FormatString, LastProcent) {
  vxlog::FormatString fs;

  fs.Init("%_ - %");
  fs.Replace("Procent");
  fs.Replace("Unused");
  EXPECT_STREQ("Procent - %", fs.GetResult());
}

TEST(FormatString, DuplicateGetResult) {
  vxlog::FormatString fs;

  fs.Init("Hello, %_ %_!");
  fs.Replace("Crazy");
  fs.GetResult();
  fs.Replace("World");
  EXPECT_STREQ("Hello, Crazy World!", fs.GetResult());
}

TEST(FormatString, ManyPlaceHolders) {
  vxlog::FormatString fs;
  fs.Init("1 %_, 2 %_, 3 %_, 4 %_, 5 %_, 6 %_");
  fs.Replace("First");
  fs.Replace("Second");
  fs.Replace("Third");
  fs.Replace("Fourth");
  fs.Replace("Fiveth");
  fs.Replace("Sixth");

  const char *expected_str =
      "1 First, 2 Second, 3 Third, 4 Fourth, 5 Fiveth, 6 Sixth";
  EXPECT_STREQ(expected_str, fs.GetResult());
}

TEST(FormatString, IntegerPlaceHolder) {
  vxlog::FormatString fs;

  fs.Init("Integer `%_'");
  fs.Replace(50);
  EXPECT_STREQ("Integer `50'", fs.GetResult());
}

TEST(FormatString, NegativeIntegerPlaceHolder) {
  vxlog::FormatString fs;

  fs.Init("Integer `%_'");
  fs.Replace(-50);
  EXPECT_STREQ("Integer `-50'", fs.GetResult());
}

TEST(FormatString, DoublePlaceHolder) {
  vxlog::FormatString fs;

  fs.Init("Double `%_'");
  fs.Replace(-50.5);
  EXPECT_STREQ("Double `-50.5'", fs.GetResult());
}

TEST(FormatString, BigString) {
  vxlog::FormatString fs;

  const size_t kUnusedSize = 1000;
  const size_t kMaxString = vxlog::FormatString::kMaxBuf + kUnusedSize;
  char str[kMaxString];
  memset(str, 'A', kMaxString);
  str[kMaxString - 1] = '\0';
  fs.Init(str);
  EXPECT_STREQ(str + kUnusedSize, fs.GetResult());
}

TEST(FormatString, PointerPlaceHolder) {
  vxlog::FormatString fs;

  fs.Init("Pointer `%_'");
  void *ptr = reinterpret_cast<void *>(0xDEADBEEF);
  fs.Replace(ptr);
  EXPECT_STREQ("Pointer `0xdeadbeef'", fs.GetResult());
}

}  // namespace

