// Copyright (c) 2009-2010 Volvox Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Author: Konstantin Lepa <konstantin.lepa@gmail.com>

#include "vxlog/type-to-string.h"
#include "utests/googletest.h"

using namespace vxlog;

namespace {

struct FixedFraction {
  FixedFraction(double n, int s) : number(n), size(s) {}

  double number;
  int    size;
};

}  // namespace

namespace vxlog {

template<>
struct TypeStringSize<FixedFraction> {
  static const size_t kValue = std::numeric_limits<double>::digits10
        + 1   // for '\0'
        + 1   // for a sign
        + 1;  // just in case
};

template<>
bool TypeToString<FixedFraction>(const FixedFraction &value,
                                 char *str, size_t len) {
  int res = snprintf(str, len, "%.*f", value.size, value.number);
  if ((res < 0) || (static_cast<size_t>(res) == len)) { return false; }
  return true;
}

}  // namespace vxlog

namespace {

TEST(TypeToString, ConvertInt32) {
  const int32_t kSampleVal = 100;
  const size_t kBufSize = TypeStringSize<int32_t>::kValue;
  char buf[kBufSize] = { '\0' };
  EXPECT_TRUE(TypeToString(kSampleVal, buf, kBufSize));
  EXPECT_STREQ("100", buf);
}

TEST(TypeToString, ConvertInt64) {
  const int64_t kSampleVal = 100;
  const size_t kBufSize = TypeStringSize<int64_t>::kValue;
  char buf[kBufSize] = { '\0' };
  EXPECT_TRUE(TypeToString(kSampleVal, buf, kBufSize));
  EXPECT_STREQ("100", buf);
}

TEST(TypeToString, ConvertSignedInt) {
  const int kSampleVal = -100;
  const size_t kBufSize = TypeStringSize<int>::kValue;
  char buf[kBufSize] = { '\0' };
  EXPECT_TRUE(TypeToString(kSampleVal, buf, kBufSize));
  EXPECT_STREQ("-100", buf);
}

TEST(TypeToString, ConvertDouble) {
  const double kSampleVal = -100.5;
  const size_t kBufSize = TypeStringSize<double>::kValue;
  char buf[kBufSize] = { '\0' };
  EXPECT_TRUE(TypeToString(kSampleVal, buf, kBufSize));
  EXPECT_STREQ("-100.5", buf);
}

TEST(TypeToString, ConvertBool) {
  const bool kSampleVal = true;
  const size_t kBufSize = TypeStringSize<bool>::kValue;
  char buf[kBufSize] = { '\0' };
  EXPECT_TRUE(TypeToString(kSampleVal, buf, kBufSize));
  EXPECT_STREQ("true", buf);
}

TEST(TypeToString, ConvertOwnType) {
  const double kSampleVal = 333.12345;
  const size_t kBufSize = TypeStringSize<FixedFraction>::kValue;
  char buf[kBufSize] = { '\0' };
  EXPECT_TRUE(TypeToString(FixedFraction(kSampleVal, 3), buf, kBufSize));
  EXPECT_STREQ("333.123", buf);
}

}  // namespace

